@@ -1,10 +1,12 @@
 from countershape import Page
 
 pages = [
+    Page("webapp.html", "Using the Web App"),
     Page("firefox.html", "Firefox"),
     Page("osx.html", "OSX"),
     Page("windows7.html", "Windows 7"),
     Page("ios.html", "IOS"),
     Page("ios-simulator.html", "IOS Simulator"),
     Page("android.html", "Android"),
+    Page("java.html", "Java"),
 ]
