@@ -61,6 +61,10 @@ class HelpView(urwid.ListBox):
                     common.highlight_key("json", "s") +
                     [("text", ": JSON")]
                 ),
+                (None,
+                    common.highlight_key("css", "c") +
+                    [("text", ": CSS")]
+                ),
                 (None,
                     common.highlight_key("urlencoded", "u") +
                     [("text", ": URL-encoded data")]
