@@ -6,7 +6,7 @@ class TestMaster:
     def test_default_handler(self):
         m = controller.Master(None)
         msg = mock.MagicMock()
-        m.handle(msg)
+        m.handle("type", msg)
         assert msg.reply.call_count == 1
 
 
