@@ -30,6 +30,9 @@ class TestDumpMaster:
         resp = tutils.tresp(req)
         resp.content = content
         m.handle_clientconnect(cc)
+        sc = proxy.ServerConnection(m.o, req.scheme, req.host, req.port, None)
+        sc.reply = mock.MagicMock()
+        m.handle_serverconnection(sc)
         m.handle_request(req)
         f = m.handle_response(resp)
         cd = flow.ClientDisconnect(cc)
@@ -43,6 +46,7 @@ class TestDumpMaster:
         m = dump.DumpMaster(None, o, filt, outfile=cs)
         for i in range(n):
             self._cycle(m, content)
+        m.shutdown()
         return cs.getvalue()
 
     def _flowfile(self, path):
@@ -110,7 +114,7 @@ class TestDumpMaster:
         o = dump.Options(app=True)
         s = mock.MagicMock()
         m = dump.DumpMaster(s, o, None)
-        assert s.apps.add.call_count == 2
+        assert len(m.apps.apps) == 1
 
     def test_replacements(self):
         o = dump.Options(replacements=[(".*", "content", "foo")])
@@ -149,19 +153,20 @@ class TestDumpMaster:
     def test_script(self):
         ret = self._dummy_cycle(
             1, None, "",
-            script=tutils.test_data.path("scripts/all.py"), verbosity=0, eventlog=True
+            scripts=[tutils.test_data.path("scripts/all.py")], verbosity=0, eventlog=True
         )
         assert "XCLIENTCONNECT" in ret
+        assert "XSERVERCONNECT" in ret
         assert "XREQUEST" in ret
         assert "XRESPONSE" in ret
         assert "XCLIENTDISCONNECT" in ret
         tutils.raises(
             dump.DumpError,
-            self._dummy_cycle, 1, None, "", script="nonexistent"
+            self._dummy_cycle, 1, None, "", scripts=["nonexistent"]
         )
         tutils.raises(
             dump.DumpError,
-            self._dummy_cycle, 1, None, "", script="starterr.py"
+            self._dummy_cycle, 1, None, "", scripts=["starterr.py"]
         )
 
     def test_stickycookie(self):
