@@ -1,10 +1,31 @@
 import Queue, time, os.path
 from cStringIO import StringIO
 import email.utils
-from libmproxy import filt, flow, controller, utils, tnetstring
+from libmproxy import filt, flow, controller, utils, tnetstring, proxy
 import tutils
 
 
+def test_app_registry():
+    ar = flow.AppRegistry()
+    ar.add("foo", "domain", 80)
+
+    r = tutils.treq()
+    r.host = "domain"
+    r.port = 80
+    assert ar.get(r)
+
+    r.port = 81
+    assert not ar.get(r)
+
+    r = tutils.treq()
+    r.host = "domain2"
+    r.port = 80
+    assert not ar.get(r)
+    r.headers["host"] = ["domain"]
+    assert ar.get(r)
+
+
+
 class TestStickyCookieState:
     def _response(self, cookie, host):
         s = flow.StickyCookieState(filt.parse(".*"))
@@ -544,9 +565,10 @@ class TestFlowMaster:
         fm = flow.FlowMaster(None, s)
         assert not fm.load_script(tutils.test_data.path("scripts/a.py"))
         assert not fm.load_script(tutils.test_data.path("scripts/a.py"))
-        assert not fm.load_script(None)
+        assert not fm.unload_scripts()
         assert fm.load_script("nonexistent")
         assert "ValueError" in fm.load_script(tutils.test_data.path("scripts/starterr.py"))
+        assert len(fm.scripts) == 0
 
     def test_replay(self):
         s = flow.State()
@@ -572,20 +594,35 @@ class TestFlowMaster:
         assert not fm.load_script(tutils.test_data.path("scripts/all.py"))
         req = tutils.treq()
         fm.handle_clientconnect(req.client_conn)
-        assert fm.script.ns["log"][-1] == "clientconnect"
+        assert fm.scripts[0].ns["log"][-1] == "clientconnect"
+        sc = proxy.ServerConnection(None, req.scheme, req.host, req.port, None)
+        sc.reply = controller.DummyReply()
+        fm.handle_serverconnection(sc)
+        assert fm.scripts[0].ns["log"][-1] == "serverconnect"
         f = fm.handle_request(req)
-        assert fm.script.ns["log"][-1] == "request"
+        assert fm.scripts[0].ns["log"][-1] == "request"
         resp = tutils.tresp(req)
         fm.handle_response(resp)
-        assert fm.script.ns["log"][-1] == "response"
+        assert fm.scripts[0].ns["log"][-1] == "response"
+        #load second script
+        assert not fm.load_script(tutils.test_data.path("scripts/all.py"))
+        assert len(fm.scripts) == 2
         dc = flow.ClientDisconnect(req.client_conn)
         dc.reply = controller.DummyReply()
         fm.handle_clientdisconnect(dc)
-        assert fm.script.ns["log"][-1] == "clientdisconnect"
+        assert fm.scripts[0].ns["log"][-1] == "clientdisconnect"
+        assert fm.scripts[1].ns["log"][-1] == "clientdisconnect"
+
+
+        #unload first script
+        fm.unload_scripts()
+        assert len(fm.scripts) == 0
+
+        assert not fm.load_script(tutils.test_data.path("scripts/all.py"))
         err = flow.Error(f.request, "msg")
         err.reply = controller.DummyReply()
         fm.handle_error(err)
-        assert fm.script.ns["log"][-1] == "error"
+        assert fm.scripts[0].ns["log"][-1] == "error"
 
     def test_duplicate_flow(self):
         s = flow.State()
