@@ -11,26 +11,6 @@ def test_proxy_error():
     assert str(p)
 
 
-def test_app_registry():
-    ar = proxy.AppRegistry()
-    ar.add("foo", "domain", 80)
-
-    r = tutils.treq()
-    r.host = "domain"
-    r.port = 80
-    assert ar.get(r)
-
-    r.port = 81
-    assert not ar.get(r)
-
-    r = tutils.treq()
-    r.host = "domain2"
-    r.port = 80
-    assert not ar.get(r)
-    r.headers["host"] = ["domain"]
-    assert ar.get(r)
-
-
 class TestServerConnection:
     def setUp(self):
         self.d = test.Daemon()
@@ -130,6 +110,7 @@ class TestProcessProxyOptions:
 
 
 class TestProxyServer:
+    @tutils.SkipWindows # binding to 0.0.0.0:1 works without special permissions on Windows
     def test_err(self):
         parser = argparse.ArgumentParser()
         cmdline.common_options(parser)
