@@ -2,11 +2,9 @@ import threading, Queue
 import flask
 import libpathod.test, libpathod.pathoc
 from libmproxy import proxy, flow, controller
+from libmproxy.cmdline import APP_HOST, APP_PORT
 import tutils
 
-APP_DOMAIN = "mitm"
-APP_IP = "1.1.1.1"
-
 testapp = flask.Flask(__name__)
 
 @testapp.route("/")
@@ -25,13 +23,13 @@ def errapp(environ, start_response):
 class TestMaster(flow.FlowMaster):
     def __init__(self, testq, config):
         s = proxy.ProxyServer(config, 0)
-        s.apps.add(testapp, "testapp", 80)
-        s.apps.add(errapp, "errapp", 80)
         state = flow.State()
         flow.FlowMaster.__init__(self, s, state)
+        self.apps.add(testapp, "testapp", 80)
+        self.apps.add(errapp, "errapp", 80)
         self.testq = testq
         self.clear_log()
-        self.start_app(APP_DOMAIN, APP_IP)
+        self.start_app(APP_HOST, APP_PORT, False)
 
     def handle_request(self, m):
         flow.FlowMaster.handle_request(self, m)
@@ -165,12 +163,12 @@ class HTTPProxTest(ProxTestBase):
         if self.ssl:
             p = libpathod.pathoc.Pathoc("127.0.0.1", self.proxy.port, True)
             print "PRE"
-            p.connect((APP_IP, 80))
+            p.connect((APP_HOST, APP_PORT))
             print "POST"
             return p.request("get:'/%s'"%page)
         else:
             p = self.pathoc()
-            return p.request("get:'http://%s/%s'"%(APP_DOMAIN, page))
+            return p.request("get:'http://%s/%s'"%(APP_HOST, page))
 
 
 class TResolver:
